@@ -1,5 +1,6 @@
 /* BFD back-end for NetBSD/m68k a.out-ish binaries.
-   Copyright (C) 1990, 1991, 1992 Free Software Foundation, Inc.
+   Copyright 1990, 1991, 1992, 1994, 1995, 1997, 1998, 2000, 2001, 2003
+   Free Software Foundation, Inc.
 
 This file is part of BFD, the Binary File Descriptor library.
 
@@ -15,18 +16,22 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
 
-#define BYTES_IN_WORD	4
 #define TARGET_IS_BIG_ENDIAN_P
 
-#define	PAGE_SIZE	4096
-#define	SEGMENT_SIZE	4096
+/* Our m68k ports use either 4K or 8K pages, but object files always
+   assume 8K page alignment so they will work on all m68k machines.  */
+#define TARGET_PAGE_SIZE	0x2000
 
 #define DEFAULT_ARCH	bfd_arch_m68k
-#define MACHTYPE_OK(mtype) ((mtype) == M_68020 || (mtype) == M_68K_NETBSD || (mtype) == M_UNKNOWN)
+#define DEFAULT_MID 	M_68K_NETBSD
+
+/* Do not "beautify" the CONCAT* macro args.  Traditional C will not
+   remove whitespace added here, and thus will fail to concatenate
+   the tokens.  */
+#define MY(OP) CONCAT2 (m68knetbsd_,OP)
 
-#define MY(OP) CAT(m68knetbsd_,OP)
 /* This needs to start with a.out so GDB knows it is an a.out variant.  */
 #define TARGETNAME "a.out-m68k-netbsd"
 
