@@ -1,5 +1,5 @@
 /* Header file for GDB CLI command implementation library.
-   Copyright (c) 2000 Free Software Foundation, Inc.
+   Copyright (c) 2000,2006 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -39,6 +39,10 @@ extern struct cmd_list_element *disablelist;
 
 extern struct cmd_list_element *deletelist;
 
+/* Chain containing all defined detach subcommands. */
+
+extern struct cmd_list_element *detachlist;
+
 /* Chain containing all defined toggle subcommands. */
 
 extern struct cmd_list_element *togglelist;
@@ -115,11 +119,16 @@ extern void cd_command (char *, int);
 
 extern void quit_command (char *, int);
 
-extern void source_command (char *, int);
+extern void source_script (char *, int);
 
 /* Used everywhere whenever at least one parameter is required and
   none is specified. */
 
 extern NORETURN void error_no_arg (char *) ATTR_NORETURN;
 
+/* Command tracing state.  */
+
+extern int source_verbose;
+extern int trace_commands;
+
 #endif /* !defined (CLI_CMDS_H) */
