@@ -29,10 +29,6 @@
 
 #define U_REGS_OFFSET 0
 
-/* The alpha does not step over a breakpoint, the manpage is lying again.  */
-
-#define CANNOT_STEP_BREAKPOINT 1
-
 /* Given a pointer to either a gregset_t or fpregset_t, return a
    pointer to the first register.  */
 #define ALPHA_REGSET_BASE(regsetp)  ((long *) (regsetp))
