@@ -29,18 +29,10 @@
 
 #define U_REGS_OFFSET 0
 
-/* FIXME: Shouldn't the default definition in inferior.h be int* ? */
-
-#define PTRACE_ARG3_TYPE int*
-
 /* ptrace transfers longs, the ptrace man page is lying.  */
 
 #define PTRACE_XFER_TYPE long
 
-/* The alpha does not step over a breakpoint, the manpage is lying again.  */
-
-#define CANNOT_STEP_BREAKPOINT 1
-
 /* Support for shared libraries.  */
 
 #include "solib.h"
