@@ -1,5 +1,5 @@
 /* Native support for GNU/Linux, for GDB, the GNU debugger.
-   Copyright (C) 1999
+   Copyright 1999, 2000
    Free Software Foundation, Inc.
 
    This file is part of GDB.
@@ -19,18 +19,20 @@
    Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.  */
 
-/* Some versions of Linux have real-time signal support in the C library, and
-   some don't.  We have to include this file to find out.  */
-#include <signal.h>
+/* Pick reasonable defaults for the number of real-time signals.  */
 
-#ifdef __SIGRTMIN
-#define REALTIME_LO __SIGRTMIN
-#define REALTIME_HI (__SIGRTMAX + 1)
-#else
+#ifndef REALTIME_LO
 #define REALTIME_LO 32
+#endif
+#ifndef REALTIME_HI
 #define REALTIME_HI 64
 #endif
 
 /* We need this file for the SOLIB_TRAMPOLINE stuff. */
 
-#include "tm-sysv4.h"
+/* If PC is in a shared library trampoline code, return the PC
+   where the function itself actually starts.  If not, return 0.  */
+
+#define SKIP_TRAMPOLINE_CODE(pc)  find_solib_trampoline_target (pc)
+
+#include "solib.h"		/* Support for shared libraries. */
