@@ -1,6 +1,6 @@
 /* Code dealing with dummy stack frames, for GDB, the GNU debugger.
 
-   Copyright 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #if !defined (DUMMY_FRAME_H)
 #define DUMMY_FRAME_H 1
@@ -27,9 +27,25 @@ struct regcache;
 struct frame_unwind;
 struct frame_id;
 
+/* Push the information needed to identify, and unwind from, a dummy
+   frame onto the dummy frame stack.  */
+
+/* NOTE: cagney/2004-08-02: This interface will eventually need to be
+   parameterized with the caller's thread - that will allow per-thread
+   dummy-frame stacks and, hence, per-thread inferior function
+   calls.  */
+
+/* NOTE: cagney/2004-08-02: In the case of ABIs using push_dummy_code
+   containing more than one instruction, this interface many need to
+   be expanded so that it knowns the lower/upper extent of the dummy
+   frame's code.  */
+
+extern void dummy_frame_push (struct regcache *regcache,
+			      const struct frame_id *dummy_id);
+
 /* If the PC falls in a dummy frame, return a dummy frame
    unwinder.  */
 
-extern const struct frame_unwind *dummy_frame_sniffer (struct frame_info *next_frame);
+extern const struct frame_unwind *const dummy_frame_unwind;
 
 #endif /* !defined (DUMMY_FRAME_H)  */
