@@ -1,5 +1,5 @@
 /* Internal interfaces for the GNU/Linux specific target code for gdbserver.
-   Copyright 2002, 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -15,8 +15,12 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
+
+#ifdef HAVE_THREAD_DB_H
+#include <thread_db.h>
+#endif
 
 #ifdef HAVE_LINUX_REGSETS
 typedef void (*regset_fill_func) (void *);
@@ -92,6 +96,9 @@ struct process_info
      event already received in a wait()).  */
   int stopped;
 
+  /* When stopped is set, the last wait status recorded for this process.  */
+  int last_status;
+
   /* If this flag is set, we have sent a SIGSTOP to this process and are
      waiting for it to stop.  */
   int sigstop_sent;
@@ -121,6 +128,11 @@ struct process_info
      and then processed and cleared in linux_resume_one_process.  */
 
   struct thread_resume *resume;
+
+#ifdef HAVE_THREAD_DB_H
+  /* The thread handle, used for e.g. TLS access.  */
+  td_thrhandle_t th;
+#endif
 };
 
 extern struct inferior_list all_processes;
@@ -128,3 +140,5 @@ extern struct inferior_list all_processes;
 void linux_attach_lwp (unsigned long pid, unsigned long tid);
 
 int thread_db_init (void);
+int thread_db_get_tls_address (struct thread_info *thread, CORE_ADDR offset,
+			       CORE_ADDR load_module, CORE_ADDR *address);
