@@ -1,6 +1,6 @@
 /* Target-dependent code for Cygwin running on i386's, for GDB.
 
-   Copyright 2003 Free Software Foundation, Inc.
+   Copyright (C) 2003 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #include "defs.h"
 #include "osabi.h"
@@ -32,19 +32,11 @@ i386_cygwin_skip_trampoline_code (CORE_ADDR pc)
   return i386_pe_skip_trampoline_code (pc, NULL);
 }
 
-static int
-i386_cygwin_in_solib_call_trampoline (CORE_ADDR pc, char *name)
-{
-  return (i386_pe_skip_trampoline_code (pc, name) != 0);
-}
-
 static void
 i386_cygwin_init_abi (struct gdbarch_info info, struct gdbarch *gdbarch)
 {
   struct gdbarch_tdep *tdep = gdbarch_tdep (gdbarch);
 
-  set_gdbarch_in_solib_call_trampoline (gdbarch,
-                                        i386_cygwin_in_solib_call_trampoline);
   set_gdbarch_skip_trampoline_code (gdbarch, i386_cygwin_skip_trampoline_code);
 
   tdep->struct_return = reg_struct_return;
