@@ -1,6 +1,6 @@
 /* Target-dependent code for the i387.
 
-   Copyright 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+   Copyright (C) 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #ifndef I387_TDEP_H
 #define I387_TDEP_H
@@ -60,13 +60,13 @@ extern void i387_print_float_info (struct gdbarch *gdbarch,
    return its contents in TO.  */
 
 extern void i387_register_to_value (struct frame_info *frame, int regnum,
-				    struct type *type, void *to);
+				    struct type *type, gdb_byte *to);
 
 /* Write the contents FROM of a value of type TYPE into register
    REGNUM in frame FRAME.  */
 
 extern void i387_value_to_register (struct frame_info *frame, int regnum,
-				    struct type *type, const void *from);
+				    struct type *type, const gdb_byte *from);
 
 
 /* Size of the memory area use by the 'fsave' and 'fxsave'
