@@ -1,6 +1,6 @@
 /* MI Command Set - output generating routines.
 
-   Copyright 2000, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 2000, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 
    Contributed by Cygnus Solutions (a Red Hat company).
 
@@ -18,8 +18,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #include "defs.h"
 #include "ui-out.h"
@@ -56,11 +56,12 @@ static void mi_field_string (struct ui_out *uiout, int fldno, int width,
 static void mi_field_fmt (struct ui_out *uiout, int fldno,
 			  int width, enum ui_align align,
 			  const char *fldname, const char *format,
-			  va_list args);
+			  va_list args) ATTR_FORMAT (printf, 6, 0);
 static void mi_spaces (struct ui_out *uiout, int numspaces);
 static void mi_text (struct ui_out *uiout, const char *string);
 static void mi_message (struct ui_out *uiout, int verbosity,
-			const char *format, va_list args);
+			const char *format, va_list args)
+     ATTR_FORMAT (printf, 3, 0);
 static void mi_wrap_hint (struct ui_out *uiout, char *identstring);
 static void mi_flush (struct ui_out *uiout);
 
@@ -98,9 +99,6 @@ static void mi_open (struct ui_out *uiout, const char *name,
 		     enum ui_out_type type);
 static void mi_close (struct ui_out *uiout, enum ui_out_type type);
 
-static void out_field_fmt (struct ui_out *uiout, int fldno, char *fldname,
-			   char *format,...);
-
 /* Mark beginning of a table */
 
 void
@@ -290,31 +288,6 @@ mi_flush (struct ui_out *uiout)
 
 /* local functions */
 
-/* Like mi_field_fmt, but takes a variable number of args
-   and makes a va_list and does not insert a separator */
-
-/* VARARGS */
-static void
-out_field_fmt (struct ui_out *uiout, int fldno, char *fldname,
-	       char *format,...)
-{
-  mi_out_data *data = ui_out_data (uiout);
-  va_list args;
-
-  field_separator (uiout);
-  if (fldname)
-    fprintf_unfiltered (data->buffer, "%s=\"", fldname);
-  else
-    fputs_unfiltered ("\"", data->buffer);
-
-  va_start (args, format);
-  vfprintf_unfiltered (data->buffer, format, args);
-
-  fputs_unfiltered ("\"", data->buffer);
-
-  va_end (args);
-}
-
 /* access to ui_out format private members */
 
 static void
@@ -346,7 +319,7 @@ mi_open (struct ui_out *uiout,
       fputc_unfiltered ('[', data->buffer);
       break;
     default:
-      internal_error (__FILE__, __LINE__, "bad switch");
+      internal_error (__FILE__, __LINE__, _("bad switch"));
     }
 }
 
@@ -364,7 +337,7 @@ mi_close (struct ui_out *uiout,
       fputc_unfiltered (']', data->buffer);
       break;
     default:
-      internal_error (__FILE__, __LINE__, "bad switch");
+      internal_error (__FILE__, __LINE__, _("bad switch"));
     }
   data->suppress_field_separator = 0;
 }
