@@ -1,5 +1,5 @@
 /* GDB routines for manipulating the minimal symbol tables.
-   Copyright 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+   Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
    2002, 2003, 2004
    Free Software Foundation, Inc.
    Contributed by Cygnus Support, using pieces from other GDB modules.
@@ -18,8 +18,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 
 /* This file contains support routines for creating, manipulating, and
@@ -145,7 +145,15 @@ add_minsym_to_demangled_hash_table (struct minimal_symbol *sym,
    Note:  One instance where there may be duplicate minimal symbols with
    the same name is when the symbol tables for a shared library and the
    symbol tables for an executable contain global symbols with the same
-   names (the dynamic linker deals with the duplication).  */
+   names (the dynamic linker deals with the duplication).
+
+   It's also possible to have minimal symbols with different mangled
+   names, but identical demangled names.  For example, the GNU C++ v3
+   ABI requires the generation of two (or perhaps three) copies of
+   constructor functions --- "in-charge", "not-in-charge", and
+   "allocate" copies; destructors may be duplicated as well.
+   Obviously, there must be distinct mangled names for each of these,
+   but the demangled names are all the same: S::S or S::~S.  */
 
 struct minimal_symbol *
 lookup_minimal_symbol (const char *name, const char *sfile,
@@ -352,7 +360,6 @@ lookup_minimal_symbol_solib_trampoline (const char *name,
   return NULL;
 }
 
-
 /* Search through the minimal symbol table for each objfile and find
    the symbol whose address is the largest address that is still less
    than or equal to PC, and matches SECTION (if non-NULL).  Returns a
@@ -404,6 +411,8 @@ lookup_minimal_symbol_by_pc_section (CORE_ADDR pc, asection *section)
 
       if (objfile->minimal_symbol_count > 0)
 	{
+	  int best_zero_sized = -1;
+
           msymbol = objfile->msymbols;
 	  lo = 0;
 	  hi = objfile->minimal_symbol_count - 1;
@@ -453,34 +462,92 @@ lookup_minimal_symbol_by_pc_section (CORE_ADDR pc, asection *section)
 			 == SYMBOL_VALUE_ADDRESS (&msymbol[hi + 1])))
 		hi++;
 
+	      /* Skip various undesirable symbols.  */
+	      while (hi >= 0)
+		{
+		  /* Skip any absolute symbols.  This is apparently
+		     what adb and dbx do, and is needed for the CM-5.
+		     There are two known possible problems: (1) on
+		     ELF, apparently end, edata, etc. are absolute.
+		     Not sure ignoring them here is a big deal, but if
+		     we want to use them, the fix would go in
+		     elfread.c.  (2) I think shared library entry
+		     points on the NeXT are absolute.  If we want
+		     special handling for this it probably should be
+		     triggered by a special mst_abs_or_lib or some
+		     such.  */
+
+		  if (msymbol[hi].type == mst_abs)
+		    {
+		      hi--;
+		      continue;
+		    }
+
+		  /* If SECTION was specified, skip any symbol from
+		     wrong section.  */
+		  if (section
+		      /* Some types of debug info, such as COFF,
+			 don't fill the bfd_section member, so don't
+			 throw away symbols on those platforms.  */
+		      && SYMBOL_BFD_SECTION (&msymbol[hi]) != NULL
+		      && (!matching_bfd_sections
+			  (SYMBOL_BFD_SECTION (&msymbol[hi]), section)))
+		    {
+		      hi--;
+		      continue;
+		    }
+
+		  /* If the minimal symbol has a zero size, save it
+		     but keep scanning backwards looking for one with
+		     a non-zero size.  A zero size may mean that the
+		     symbol isn't an object or function (e.g. a
+		     label), or it may just mean that the size was not
+		     specified.  */
+		  if (MSYMBOL_SIZE (&msymbol[hi]) == 0
+		      && best_zero_sized == -1)
+		    {
+		      best_zero_sized = hi;
+		      hi--;
+		      continue;
+		    }
+
+		  /* Otherwise, this symbol must be as good as we're going
+		     to get.  */
+		  break;
+		}
+
+	      /* If HI has a zero size, and best_zero_sized is set,
+		 then we had two or more zero-sized symbols; prefer
+		 the first one we found (which may have a higher
+		 address).  Also, if we ran off the end, be sure
+		 to back up.  */
+	      if (best_zero_sized != -1
+		  && (hi < 0 || MSYMBOL_SIZE (&msymbol[hi]) == 0))
+		hi = best_zero_sized;
+
+	      /* If the minimal symbol has a non-zero size, and this
+		 PC appears to be outside the symbol's contents, then
+		 refuse to use this symbol.  If we found a zero-sized
+		 symbol with an address greater than this symbol's,
+		 use that instead.  We assume that if symbols have
+		 specified sizes, they do not overlap.  */
+
+	      if (hi >= 0
+		  && MSYMBOL_SIZE (&msymbol[hi]) != 0
+		  && pc >= (SYMBOL_VALUE_ADDRESS (&msymbol[hi])
+			    + MSYMBOL_SIZE (&msymbol[hi])))
+		{
+		  if (best_zero_sized != -1)
+		    hi = best_zero_sized;
+		  else
+		    /* Go on to the next object file.  */
+		    continue;
+		}
+
 	      /* The minimal symbol indexed by hi now is the best one in this
 	         objfile's minimal symbol table.  See if it is the best one
 	         overall. */
 
-	      /* Skip any absolute symbols.  This is apparently what adb
-	         and dbx do, and is needed for the CM-5.  There are two
-	         known possible problems: (1) on ELF, apparently end, edata,
-	         etc. are absolute.  Not sure ignoring them here is a big
-	         deal, but if we want to use them, the fix would go in
-	         elfread.c.  (2) I think shared library entry points on the
-	         NeXT are absolute.  If we want special handling for this
-	         it probably should be triggered by a special
-	         mst_abs_or_lib or some such.  */
-	      while (hi >= 0
-		     && msymbol[hi].type == mst_abs)
-		--hi;
-
-	      /* If "section" specified, skip any symbol from wrong section */
-	      /* This is the new code that distinguishes it from the old function */
-	      if (section)
-		while (hi >= 0
-		       /* Some types of debug info, such as COFF,
-			  don't fill the bfd_section member, so don't
-			  throw away symbols on those platforms.  */
-		       && SYMBOL_BFD_SECTION (&msymbol[hi]) != NULL
-		       && SYMBOL_BFD_SECTION (&msymbol[hi]) != section)
-		  --hi;
-
 	      if (hi >= 0
 		  && ((best_symbol == NULL) ||
 		      (SYMBOL_VALUE_ADDRESS (best_symbol) <
