@@ -1,5 +1,6 @@
 /* Pascal language support definitions for GDB, the GNU debugger.
-   Copyright 2000 Free Software Foundation, Inc.
+
+   Copyright (C) 2000, 2005 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -15,7 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 /* This file is derived from c-lang.h */
 
@@ -28,7 +30,8 @@ extern void pascal_error (char *);	/* Defined in p-exp.y */
 /* Defined in p-typeprint.c */
 extern void pascal_print_type (struct type *, char *, struct ui_file *, int, int);
 
-extern int pascal_val_print (struct type *, char *, int, CORE_ADDR, struct ui_file *, int, int,
+extern int pascal_val_print (struct type *, const gdb_byte *, int,
+			     CORE_ADDR, struct ui_file *, int, int,
 			     int, enum val_prettyprint);
 
 extern int pascal_value_print (struct value *, struct ui_file *, int, enum val_prettyprint);
@@ -38,11 +41,13 @@ extern void pascal_type_print_method_args (char *, char *,
 
 /* These are in p-lang.c: */
 
-extern int is_pascal_string_type (struct type *, int *, int *, int *, int*);
+extern int 
+  is_pascal_string_type (struct type *, int *, int *, int *, int *, char **);
 
 extern void pascal_printchar (int, struct ui_file *);
 
-extern void pascal_printstr (struct ui_file *, char *, unsigned int, int, int);
+extern void pascal_printstr (struct ui_file *, const gdb_byte *,
+			     unsigned int, int, int);
 
 extern struct type *pascal_create_fundamental_type (struct objfile *, int);
 
@@ -62,12 +67,15 @@ extern int vtblprint;		/* Controls printing of vtbl's */
 
 extern int static_field_print;
 
-extern void pascal_object_print_class_member (char *, struct type *, struct ui_file *, char *);
+extern void pascal_object_print_class_member (const gdb_byte *, struct type *,
+					      struct ui_file *, char *);
 
-extern void pascal_object_print_class_method (char *, struct type *, struct ui_file *);
+extern void pascal_object_print_class_method (const gdb_byte *, struct type *,
+					      struct ui_file *);
 
-extern void pascal_object_print_value_fields (struct type *, char *, CORE_ADDR,
-			   struct ui_file *, int, int, enum val_prettyprint,
+extern void pascal_object_print_value_fields (struct type *, const gdb_byte *,
+					      CORE_ADDR, struct ui_file *,
+					      int, int, enum val_prettyprint,
 					      struct type **, int);
 
 extern int pascal_object_is_vtbl_ptr_type (struct type *);
