@@ -1,6 +1,7 @@
 /* Scheme/Guile language support routines for GDB, the GNU debugger.
 
-   Copyright 1995, 1996, 2000, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 2000, 2003, 2005 Free Software Foundation,
+   Inc.
 
    This file is part of GDB.
 
@@ -16,8 +17,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #include "defs.h"
 #include "symtab.h"
@@ -310,10 +311,10 @@ tryagain:
       if (!skipping)
 	{
 	  struct value *val = scm_evaluate_string (str.ptr, lexptr - str.ptr);
-	  if (!is_scmvalue_type (VALUE_TYPE (val)))
+	  if (!is_scmvalue_type (value_type (val)))
 	    error ("quoted scm form yields non-SCM value");
-	  svalue = extract_signed_integer (VALUE_CONTENTS (val),
-					   TYPE_LENGTH (VALUE_TYPE (val)));
+	  svalue = extract_signed_integer (value_contents (val),
+					   TYPE_LENGTH (value_type (val)));
 	  goto handle_immediate;
 	}
       return;
