@@ -1,5 +1,5 @@
 /* Remote serial interface for local (hardwired) serial ports for GO32.
-   Copyright 1992, 1993, 2000, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1992, 1993, 2000, 2001 Free Software Foundation, Inc.
 
    Contributed by Nigel Stephens, Algorithmics Ltd. (nigel@algor.co.uk).
 
@@ -20,8 +20,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #include "defs.h"
 #include "gdbcmd.h"
@@ -903,62 +903,62 @@ _initialize_ser_dos (void)
     intrupts[1].inuse =		/* keyboard */
     intrupts[2].inuse = 1;	/* slave icu */
 
-  add_show_from_set (
-		      add_set_cmd ("com1base", class_obscure, var_zinteger,
-				   (char *) &ports[0].base,
-				   "Set COM1 base i/o port address.",
-				   &setlist),
-		      &showlist);
+  add_setshow_zinteger_cmd ("com1base", class_obscure, &ports[0].base, _("\
+Set COM1 base i/o port address."), _("\
+Show COM1 base i/o port address."), NULL,
+			    NULL,
+			    NULL, /* FIXME: i18n: */
+			    &setlist, &showlist);
 
-  add_show_from_set (
-		      add_set_cmd ("com1irq", class_obscure, var_zinteger,
-				   (char *) &ports[0].irq,
-				   "Set COM1 interrupt request.",
-				   &setlist),
-		      &showlist);
+  add_setshow_zinteger_cmd ("com1irq", class_obscure, &ports[0].irq, _("\
+Set COM1 interrupt request."), _("\
+Show COM1 interrupt request."), NULL,
+			    NULL,
+			    NULL, /* FIXME: i18n: */
+			    &setlist, &showlist);
 
-  add_show_from_set (
-		      add_set_cmd ("com2base", class_obscure, var_zinteger,
-				   (char *) &ports[1].base,
-				   "Set COM2 base i/o port address.",
-				   &setlist),
-		      &showlist);
+  add_setshow_zinteger_cmd ("com2base", class_obscure, &ports[1].base, _("\
+Set COM2 base i/o port address."), _("\
+Show COM2 base i/o port address."), NULL,
+			    NULL,
+			    NULL, /* FIXME: i18n: */
+			    &setlist, &showlist);
 
-  add_show_from_set (
-		      add_set_cmd ("com2irq", class_obscure, var_zinteger,
-				   (char *) &ports[1].irq,
-				   "Set COM2 interrupt request.",
-				   &setlist),
-		      &showlist);
+  add_setshow_zinteger_cmd ("com2irq", class_obscure, &ports[1].irq, _("\
+Set COM2 interrupt request."), _("\
+Show COM2 interrupt request."), NULL,
+			    NULL,
+			    NULL, /* FIXME: i18n: */
+			    &setlist, &showlist);
 
-  add_show_from_set (
-		      add_set_cmd ("com3base", class_obscure, var_zinteger,
-				   (char *) &ports[2].base,
-				   "Set COM3 base i/o port address.",
-				   &setlist),
-		      &showlist);
+  add_setshow_zinteger_cmd ("com3base", class_obscure, &ports[2].base, _("\
+Set COM3 base i/o port address."), _("\
+Show COM3 base i/o port address."), NULL,
+			    NULL,
+			    NULL, /* FIXME: i18n: */
+			    &setlist, &showlist);
 
-  add_show_from_set (
-		      add_set_cmd ("com3irq", class_obscure, var_zinteger,
-				   (char *) &ports[2].irq,
-				   "Set COM3 interrupt request.",
-				   &setlist),
-		      &showlist);
+  add_setshow_zinteger_cmd ("com3irq", class_obscure, &ports[2].irq, _("\
+Set COM3 interrupt request."), _("\
+Show COM3 interrupt request."), NULL,
+			    NULL,
+			    NULL, /* FIXME: i18n: */
+			    &setlist, &showlist);
 
-  add_show_from_set (
-		      add_set_cmd ("com4base", class_obscure, var_zinteger,
-				   (char *) &ports[3].base,
-				   "Set COM4 base i/o port address.",
-				   &setlist),
-		      &showlist);
+  add_setshow_zinteger_cmd ("com4base", class_obscure, &ports[3].base, _("\
+Set COM4 base i/o port address."), _("\
+Show COM4 base i/o port address."), NULL,
+			    NULL,
+			    NULL, /* FIXME: i18n: */
+			    &setlist, &showlist);
 
-  add_show_from_set (
-		      add_set_cmd ("com4irq", class_obscure, var_zinteger,
-				   (char *) &ports[3].irq,
-				   "Set COM4 interrupt request.",
-				   &setlist),
-		      &showlist);
+  add_setshow_zinteger_cmd ("com4irq", class_obscure, &ports[3].irq, _("\
+Set COM4 interrupt request."), _("\
+Show COM4 interrupt request."), NULL,
+			    NULL,
+			    NULL, /* FIXME: i18n: */
+			    &setlist, &showlist);
 
   add_info ("serial", dos_info,
-	    "Print DOS serial port status.");
+	    _("Print DOS serial port status."));
 }
