@@ -1,5 +1,5 @@
-/* Remote target glue for the Hitachi SH-3 ROM monitor.
-   Copyright 1995, 1996, 1997, 1998, 1999, 2000, 2001
+/* Remote target glue for the Renesas SH-3 ROM monitor.
+   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001
    Free Software Foundation, Inc.
 
    This file is part of GDB.
@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #include "defs.h"
 #include "gdbcore.h"
@@ -77,9 +77,9 @@ sh3_supply_register (char *regname, int regnamelen, char *val, int vallen)
 	  break;
 	case 'S':
 	  if (regname[1] == 'S' && regname[2] == 'R')
-	    regno = gdbarch_tdep (current_gdbarch)->SSR_REGNUM;
+	    regno = SSR_REGNUM;
 	  else if (regname[1] == 'P' && regname[2] == 'C')
-	    regno = gdbarch_tdep (current_gdbarch)->SPC_REGNUM;
+	    regno = SPC_REGNUM;
 	  break;
 	}
     }
@@ -228,7 +228,7 @@ init_sh3_cmds (void)
   sh3_cmds.getreg.term_cmd = ".\r";	/* getreg.term_cmd */
   sh3_cmds.dump_registers = "r\r";	/* dump_registers */
   sh3_cmds.register_pattern = "\\(\\w+\\)=\\([0-9a-fA-F]+\\( +[0-9a-fA-F]+\\b\\)*\\)";
-  sh3_cmds.supply_register = sh3_supply_register;	/* supply_register */
+  sh3_cmds.supply_register = sh3_supply_register;
   sh3_cmds.load_routine = sh3_load;	/* load_routine */
   sh3_cmds.load = NULL;		/* download command */
   sh3_cmds.loadresp = NULL;	/* Load response */
@@ -276,13 +276,20 @@ sh3_open (char *args, int from_tty)
       parallel = serial_open (parallel_port_name);
 
       if (!parallel)
-	perror_with_name ("Unable to open parallel port.");
+	perror_with_name (_("Unable to open parallel port."));
 
       parallel_in_use = 1;
     }
 
+
   /* If we connected successfully, we know the processor is an SH3.  */
-  set_architecture_from_arch_mach (bfd_arch_sh, bfd_mach_sh3);
+  {
+    struct gdbarch_info info;
+    gdbarch_info_init (&info);
+    info.bfd_arch_info = bfd_lookup_arch (bfd_arch_sh, bfd_mach_sh3);
+    if (!gdbarch_update_p (info))
+      error (_("Target is not an SH3"));
+  }
 }
 
 
@@ -323,13 +330,19 @@ sh3e_open (char *args, int from_tty)
       parallel = serial_open (parallel_port_name);
 
       if (!parallel)
-	perror_with_name ("Unable to open parallel port.");
+	perror_with_name (_("Unable to open parallel port."));
 
       parallel_in_use = 1;
     }
 
   /* If we connected successfully, we know the processor is an SH3E.  */
-  set_architecture_from_arch_mach (bfd_arch_sh, bfd_mach_sh3);
+  {
+    struct gdbarch_info info;
+    gdbarch_info_init (&info);
+    info.bfd_arch_info = bfd_lookup_arch (bfd_arch_sh, bfd_mach_sh3);
+    if (!gdbarch_update_p (info))
+      error (_("Target is not an SH3"));
+  }
 }
 
 static void
@@ -352,11 +365,11 @@ _initialize_sh3_rom (void)
   init_monitor_ops (&sh3_ops);
 
   sh3_ops.to_shortname = "sh3";
-  sh3_ops.to_longname = "Hitachi SH-3 rom monitor";
+  sh3_ops.to_longname = "Renesas SH-3 rom monitor";
 
   sh3_ops.to_doc =
   /* We can download through the parallel port too. */
-    "Debug on a Hitachi eval board running the SH-3E rom monitor.\n"
+    "Debug on a Renesas eval board running the SH-3E rom monitor.\n"
     "Specify the serial device it is connected to.\n"
     "If you want to use the parallel port to download to it, specify that\n"
     "as an additional second argument.";
@@ -371,11 +384,11 @@ _initialize_sh3_rom (void)
   init_monitor_ops (&sh3e_ops);
 
   sh3e_ops.to_shortname = "sh3e";
-  sh3e_ops.to_longname = "Hitachi SH-3E rom monitor";
+  sh3e_ops.to_longname = "Renesas SH-3E rom monitor";
 
   sh3e_ops.to_doc =
   /* We can download through the parallel port too. */
-    "Debug on a Hitachi eval board running the SH-3E rom monitor.\n"
+    "Debug on a Renesas eval board running the SH-3E rom monitor.\n"
     "Specify the serial device it is connected to.\n"
     "If you want to use the parallel port to download to it, specify that\n"
     "as an additional second argument.";
