@@ -1,6 +1,6 @@
 /* Target-dependent code for OpenBSD/sparc.
 
-   Copyright 2004 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -16,21 +16,25 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #include "defs.h"
 #include "floatformat.h"
 #include "frame.h"
 #include "frame-unwind.h"
+#include "gdbcore.h"
 #include "osabi.h"
-#include "solib-svr4.h"
+#include "regcache.h"
 #include "symtab.h"
 #include "trad-frame.h"
 
 #include "gdb_assert.h"
 
+#include "obsd-tdep.h"
 #include "sparc-tdep.h"
+#include "solib-svr4.h"
+#include "bsd-uthread.h"
 
 /* Signal trampolines.  */
 
@@ -142,6 +146,88 @@ sparc32obsd_sigtramp_frame_sniffer (struct frame_info *next_frame)
 }
 
 
+/* Offset wthin the thread structure where we can find %fp and %i7.  */
+#define SPARC32OBSD_UTHREAD_FP_OFFSET	128
+#define SPARC32OBSD_UTHREAD_PC_OFFSET	132
+
+static void
+sparc32obsd_supply_uthread (struct regcache *regcache,
+			    int regnum, CORE_ADDR addr)
+{
+  CORE_ADDR fp, fp_addr = addr + SPARC32OBSD_UTHREAD_FP_OFFSET;
+  gdb_byte buf[4];
+
+  gdb_assert (regnum >= -1);
+
+  fp = read_memory_unsigned_integer (fp_addr, 4);
+  if (regnum == SPARC_SP_REGNUM || regnum == -1)
+    {
+      store_unsigned_integer (buf, 4, fp);
+      regcache_raw_supply (regcache, SPARC_SP_REGNUM, buf);
+
+      if (regnum == SPARC_SP_REGNUM)
+	return;
+    }
+
+  if (regnum == SPARC32_PC_REGNUM || regnum == SPARC32_NPC_REGNUM
+      || regnum == -1)
+    {
+      CORE_ADDR i7, i7_addr = addr + SPARC32OBSD_UTHREAD_PC_OFFSET;
+
+      i7 = read_memory_unsigned_integer (i7_addr, 4);
+      if (regnum == SPARC32_PC_REGNUM || regnum == -1)
+	{
+	  store_unsigned_integer (buf, 4, i7 + 8);
+	  regcache_raw_supply (regcache, SPARC32_PC_REGNUM, buf);
+	}
+      if (regnum == SPARC32_NPC_REGNUM || regnum == -1)
+	{
+	  store_unsigned_integer (buf, 4, i7 + 12);
+	  regcache_raw_supply (regcache, SPARC32_NPC_REGNUM, buf);
+	}
+
+      if (regnum == SPARC32_PC_REGNUM || regnum == SPARC32_NPC_REGNUM)
+	return;
+    }
+
+  sparc_supply_rwindow (regcache, fp, regnum);
+}
+
+static void
+sparc32obsd_collect_uthread(const struct regcache *regcache,
+			    int regnum, CORE_ADDR addr)
+{
+  CORE_ADDR sp;
+  gdb_byte buf[4];
+
+  gdb_assert (regnum >= -1);
+
+  if (regnum == SPARC_SP_REGNUM || regnum == -1)
+    {
+      CORE_ADDR fp_addr = addr + SPARC32OBSD_UTHREAD_FP_OFFSET;
+
+      regcache_raw_collect (regcache, SPARC_SP_REGNUM, buf);
+      write_memory (fp_addr,buf, 4);
+    }
+
+  if (regnum == SPARC32_PC_REGNUM || regnum == -1)
+    {
+      CORE_ADDR i7, i7_addr = addr + SPARC32OBSD_UTHREAD_PC_OFFSET;
+
+      regcache_raw_collect (regcache, SPARC32_PC_REGNUM, buf);
+      i7 = extract_unsigned_integer (buf, 4) - 8;
+      write_memory_unsigned_integer (i7_addr, 4, i7);
+
+      if (regnum == SPARC32_PC_REGNUM)
+	return;
+    }
+
+  regcache_raw_collect (regcache, SPARC_SP_REGNUM, buf);
+  sp = extract_unsigned_integer (buf, 4);
+  sparc_collect_rwindow (regcache, sp, regnum);
+}
+
+
 static void
 sparc32obsd_init_abi (struct gdbarch_info info, struct gdbarch *gdbarch)
 {
@@ -150,7 +236,13 @@ sparc32obsd_init_abi (struct gdbarch_info info, struct gdbarch *gdbarch)
   /* OpenBSD/sparc is very similar to NetBSD/sparc ELF.  */
   sparc32nbsd_elf_init_abi (info, gdbarch);
 
+  set_gdbarch_skip_solib_resolver (gdbarch, obsd_skip_solib_resolver);
+
   frame_unwind_append_sniffer (gdbarch, sparc32obsd_sigtramp_frame_sniffer);
+
+  /* OpenBSD provides a user-level threads implementation.  */
+  bsd_uthread_set_supply_uthread (gdbarch, sparc32obsd_supply_uthread);
+  bsd_uthread_set_collect_uthread (gdbarch, sparc32obsd_collect_uthread);
 }
 
 
