@@ -1,5 +1,5 @@
 /* Data structures associated with tracepoints in GDB.
-   Copyright 1997, 1998, 1999, 2000 Free Software Foundation, Inc.
+   Copyright (C) 1997, 1998, 1999, 2000 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #if !defined (TRACEPOINT_H)
 #define TRACEPOINT_H 1
@@ -37,39 +37,44 @@ struct tracepoint
     int enabled_p;
 
 #if 0
-    /* Type of tracepoint (MVS FIXME: needed?). */
+    /* Type of tracepoint.  (MVS FIXME: needed?) */
     enum tptype type;
 
-    /* What to do with this tracepoint after we hit it MVS FIXME: needed?). */
+    /* What to do with this tracepoint after we hit it 
+       MVS FIXME: needed?).  */
     enum tpdisp disposition;
 #endif
     /* Number assigned to distinguish tracepoints.  */
     int number;
 
-    /* Address to trace at, or NULL if not an instruction tracepoint (MVS ?). */
+    /* Address to trace at, or NULL if not an instruction tracepoint.
+       (MVS ?) */
     CORE_ADDR address;
 
-    /* Line number of this address.  Only matters if address is non-NULL.  */
+    /* Line number of this address.  
+       Only matters if address is non-NULL.  */
     int line_number;
 
-    /* Source file name of this address.  Only matters if address is non-NULL. */
+    /* Source file name of this address.  
+       Only matters if address is non-NULL.  */
     char *source_file;
 
     /* Number of times this tracepoint should single-step 
-       and collect additional data */
+       and collect additional data.  */
     long step_count;
 
-    /* Number of times this tracepoint should be hit before disabling/ending. */
+    /* Number of times this tracepoint should be hit before 
+       disabling/ending.  */
     int pass_count;
 
-    /* Chain of action lines to execute when this tracepoint is hit. */
+    /* Chain of action lines to execute when this tracepoint is hit.  */
     struct action_line *actions;
 
     /* Conditional (MVS ?).  */
     struct expression *cond;
 
-    /* String we used to set the tracepoint (malloc'd).  Only matters if
-       address is non-NULL.  */
+    /* String we used to set the tracepoint (malloc'd).
+       Only matters if address is non-NULL.  */
     char *addr_string;
 
     /* Language we used to set the tracepoint.  */
@@ -84,11 +89,12 @@ struct tracepoint
        aborting, so you can back up to just before the abort.  */
     int hit_count;
 
-    /* Thread number for thread-specific tracepoint, or -1 if don't care */
+    /* Thread number for thread-specific tracepoint, 
+       or -1 if don't care.  */
     int thread;
 
-    /* BFD section, in case of overlays: 
-       no, I don't know if tracepoints are really gonna work with overlays.  */
+    /* BFD section, in case of overlays: no, I don't know if
+       tracepoints are really gonna work with overlays.  */
     asection *section;
   };
 
@@ -101,19 +107,19 @@ enum actionline_type
   };
 
 
-/* The tracepoint chain of all tracepoints */
+/* The tracepoint chain of all tracepoints.  */
 
 extern struct tracepoint *tracepoint_chain;
 
 extern unsigned long trace_running_p;
 
-/* A hook used to notify the UI of tracepoint operations */
+/* A hook used to notify the UI of tracepoint operations.  */
 
-void (*create_tracepoint_hook) (struct tracepoint *);
-void (*delete_tracepoint_hook) (struct tracepoint *);
-void (*modify_tracepoint_hook) (struct tracepoint *);
-void (*trace_find_hook) (char *arg, int from_tty);
-void (*trace_start_stop_hook) (int start, int from_tty);
+void (*deprecated_create_tracepoint_hook) (struct tracepoint *);
+void (*deprecated_delete_tracepoint_hook) (struct tracepoint *);
+void (*deprecated_modify_tracepoint_hook) (struct tracepoint *);
+void (*deprecated_trace_find_hook) (char *arg, int from_tty);
+void (*deprecated_trace_start_stop_hook) (int start, int from_tty);
 
 struct tracepoint *get_tracepoint_by_number (char **, int, int);
 int get_traceframe_number (void);
@@ -122,8 +128,8 @@ enum actionline_type validate_actionline (char **, struct tracepoint *);
 
 
 /* Walk the following statement or block through all tracepoints.
-   ALL_TRACEPOINTS_SAFE does so even if the statment deletes the current
-   breakpoint.  */
+   ALL_TRACEPOINTS_SAFE does so even if the statment deletes the
+   current breakpoint.  */
 
 #define ALL_TRACEPOINTS(t)  for (t = tracepoint_chain; t; t = t->next)
 
@@ -131,4 +137,4 @@ enum actionline_type validate_actionline (char **, struct tracepoint *);
 	for (t = tracepoint_chain;	\
 	     t ? (tmp = t->next, 1) : 0;\
 	     t = tmp)
-#endif /* TRACEPOINT_H */
+#endif	/* TRACEPOINT_H */
