@@ -1,5 +1,6 @@
-/* Longjump free calls to gdb internal routines.
-   Copyright 1999, 2000 Free Software Foundation, Inc.
+/* Longjump free calls to GDB internal routines.
+
+   Copyright (C) 1999, 2000, 2005 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,20 +14,18 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #ifndef WRAPPER_H
 #define WRAPPER_H 1
+
 #include "gdb.h"
 
 struct value;
 struct expression;
 struct block;
 
-/* Use this struct to pass arguments to wrapper routines. */
-struct gdb_wrapper_arguments;
-
 extern int gdb_parse_exp_1 (char **, struct block *,
 			    int, struct expression **);
 
@@ -38,14 +37,17 @@ extern int gdb_value_equal (struct value *, struct value *, int *);
 
 extern int gdb_value_assign (struct value *, struct value *, struct value **);
 
-extern int gdb_value_subscript (struct value *, struct value *, struct value **);
+extern int gdb_value_subscript (struct value *, struct value *,
+				struct value **);
 
-extern enum gdb_rc gdb_value_struct_elt (struct ui_out *uiout, struct value **result_ptr,
-					 struct value **argp, struct value **args,
-					 char *name, int *static_memfuncp, char *err);
+extern enum gdb_rc gdb_value_struct_elt (struct ui_out *uiout,
+					 struct value **result_ptr,
+					 struct value **argp,
+					 struct value **args, char *name,
+					 int *static_memfuncp, char *err);
 
 extern int gdb_value_ind (struct value *val, struct value ** rval);
 
 extern int gdb_parse_and_eval_type (char *, int, struct type **);
 
-#endif /* WRAPPER_H */
+#endif /* wrapper.h */
